#!/bin/bash -u

# Copyright (c) 2011-2016, The Ohio State University. All rights reserved.
#
# This file is part of the RDMA for Apache Hadoop software package
# developed by the team members of The Ohio State University's
# Network-Based Computing Laboratory (NBCL), headed by Professor
# Dhabaleswar K. (DK) Panda.
#
# For detailed copyright and licensing information, please refer to
# the license file LICENSE.txt in the top level directory. 


# Script to install the plugin

usage() {
	logAndExit "Usage: `basename $0` HADOOP_SRC_DIR DISTRIBUTION"
}

logAndExit() {
	echo -e "\n$1"
	echo -e "exiting...\n"
	exit 1
}

revertPatch() {
	echo -e "\nReverting applied patch.."
	sed 's#PLUGIN_JAR_DIR#'$JAR_DIR'#g' "$PATCH_DIR/"*".patch" | patch -p1 -R --silent --directory=$HADOOP_ROOT
}

if [[ "${#}" -lt 2 ]]; then
	usage
fi

PLUGIN_DIR="$(dirname -- "$(readlink -f -- "$0")")"
HADOOP_ROOT=`readlink -m "$1"`
HADOOP_DIST="$2"
APACHE_VERSION="2.7.3"
HDP_VERSION="2.5.0.3"
CDH_VERSION="5.8.2"

if [[ $HADOOP_DIST == "apache" ]]; then
	echo -e "Installing patch for $HADOOP_DIST"
	VERSION=$APACHE_VERSION
elif [[ $HADOOP_DIST == "hdp" ]]; then
	echo -e "Installing patch for $HADOOP_DIST"
	VERSION=$HDP_VERSION
elif [[ $HADOOP_DIST == "cdh" ]]; then
	echo -e "Installing patch for $HADOOP_DIST"
	VERSION=$CDH_VERSION
else
	logAndExit "Unknown hadoop distribution"
fi

PATCH_DIR="${PLUGIN_DIR}/patch/${HADOOP_DIST}/${VERSION}"
JAR_DIR="${PLUGIN_DIR}/jar/${HADOOP_DIST}/${VERSION}"
LIB_DIR="${PLUGIN_DIR}/lib/${HADOOP_DIST}/${VERSION}"
TXT_DIR="${PLUGIN_DIR}/txt/${HADOOP_DIST}/${VERSION}"
BIN_DIR="${PLUGIN_DIR}/bin"

PLUGIN_TAR="rdma-hadoop-2.x-1.1.0-bin.tar.gz"

if [[ ! -d $HADOOP_ROOT ]]; then
	logAndExit "Missing hadoop root dir: $HADOOP_ROOT"
fi

echo -e "\nHadoop src dir: $HADOOP_ROOT"

# apply patch to hadoop root
sed 's#PLUGIN_JAR_DIR#'$JAR_DIR'#g' "$PATCH_DIR/"*".patch" | patch -p1 -N --dry-run --silent --directory=$HADOOP_ROOT
if [[ $? -ne 0 ]]; then
	logAndExit "Problem occurred while trying to apply the patch.."
fi

echo -e "\nPatching files....\n"
sed 's#PLUGIN_JAR_DIR#'$JAR_DIR'#g' "$PATCH_DIR/"*".patch" | patch -p1 -N --directory=$HADOOP_ROOT

# compile hadoop
echo -e "\nCompiling hadoop..\n"
mvn -f $HADOOP_ROOT/pom.xml clean package -Pdist,native -DskipTests -Dtar
if [[ $? -ne 0 ]]; then
	revertPatch
	logAndExit "Problem occurred while compiling hadoop, and revert the applied patch.."
fi

# get version from pom.xml
SUFFIX=`mvn org.apache.maven.plugins:maven-help-plugin:2.1.1:evaluate -Dexpression=project.version -f $HADOOP_ROOT/hadoop-dist/pom.xml | grep -Ev '(^\[|Download\w+:)'`
if [[ $? -ne 0 ]]; then
	revertPatch
	logAndExit "Problem occurred while executing mvn command.."
fi

echo -e "\nGet the version number: $SUFFIX\n"

# copy jars to appropriate localtion
HADOOP_DIR="$HADOOP_ROOT/hadoop-dist/target/hadoop-$SUFFIX"
echo -e "\nCopying rdma jars..\n"
cp -v "$JAR_DIR/hadoop-common-rdma-${VERSION}.jar" "$HADOOP_DIR/share/hadoop/common"
cp -v "$JAR_DIR/hadoop-hdfs-rdma-${VERSION}.jar" "$HADOOP_DIR/share/hadoop/hdfs"
cp -v "$JAR_DIR/hadoop-mapreduce-client-app-rdma-${VERSION}.jar" "$HADOOP_DIR/share/hadoop/mapreduce"
cp -v "$JAR_DIR/hadoop-mapreduce-client-core-rdma-${VERSION}.jar" "$HADOOP_DIR/share/hadoop/mapreduce"
cp -v "$JAR_DIR/hadoop-mapreduce-client-shuffle-rdma-${VERSION}.jar" "$HADOOP_DIR/share/hadoop/mapreduce"
cp -v "$JAR_DIR/hadoop-yarn-server-nodemanager-rdma-${VERSION}.jar" "$HADOOP_DIR/share/hadoop/yarn"
if [[ $HADOOP_DIST == "cdh" ]]; then
  cp -v "$JAR_DIR/hadoop-yarn-server-resourcemanager-rdma-${VERSION}.jar" "$HADOOP_DIR/share/hadoop/yarn"
fi

# copy rdma libs
echo -e "\nCopying rdma libs..\n"
cp -v "$LIB_DIR/"* "$HADOOP_DIR/lib/native"
ldconfig -n "$HADOOP_DIR/lib/native"

# copy slurm-pbs scripts
echo -e "\nCopying slurm-pbs scripts..\n"
cp -vrf "$BIN_DIR/"* "$HADOOP_DIR/bin/"

# copy txt files
cp -v "${TXT_DIR}/"*".txt" "$HADOOP_DIR"

# tar the plugin hadoop and print the localtion
echo -e "\nCretaing tar.gz.."
cd "$HADOOP_ROOT/hadoop-dist/target"
tar czf "$PLUGIN_TAR" "hadoop-$SUFFIX"
if [[ $? -ne 0 ]]; then
	cd $PLUGIN_DIR
	revertPatch
	logAndExit "Problem occurred while creating tar.."
fi

echo -e "\nPlugin is applied successfully.."
echo -e "\nRDMA-plugin based Hadoop dist tar is available at: $HADOOP_ROOT/hadoop-dist/target/$PLUGIN_TAR"

cd -
revertPatch
exit 0
